#! /bin/sh

PATCH=qsearch-hooks.patch

echo "Not working yet..."

function error
{
	echo "ERROR: $1"
	exit
}


KERNEL_DIR=${1:-/usr/src/linux}
SRC_DIR=${0%/*}
PLUGINS_DIR=$SRC_DIR/../plugins
TMPL_DIR=$SRC_DIR/templates


# Testing parameters 
[ -f "$KERNEL_DIR/kernel/printk.c" ] || error "$KERNEL_DIR is not a kernel source directory."
[ -f "$SRC_DIR/$PATCH" ] || error "$SRC_DIR does not contain $PATCH"
[ -d "$SRC_DIR/qsearch" ] || error "$SRC_DIR does not contain qsearch directory."
[ -d "$PLUGINS_DIR" ] || error "$PLUGINS_DIR does not exist."

# Patching kernel source directory
echo "######### Patching $KERNEL_DIR with $PATCH..."
( cd $KERNEL_DIR && patch -p1 ) < $SRC_DIR/$PATCH

# Installing kernel implementation of the API
cp -a $SRC_DIR/qsearch $KERNEL_DIR
cp $SRC_DIR/../src/include/qsearch.h $KERNEL_DIR/include


# Find plugins
SUBDIRS=`grep SUBDIRS $PLUGINS_DIR/Makefile.am`
PLUGINS=${SUBDIRS#*=}
echo "Found plugins : [$PLUGINS]"

# Create Makefile
( cat $TMPL_DIR/Makefile.part1
  for i in $PLUGINS; 
  do 
    UPCASE=`echo "$i" | tr [:lower:] [:upper:]`
    echo "subdir-\$(CONFIG_QSEARCH_$UPCASE)        += $i"
  done
  cat $TMPL_DIR/Makefile.part2
) > $KERNEL_DIR/qsearch/Makefile

# Create Config.in
( cat $TMPL_DIR/Config.in.part1
  for i in $PLUGINS
  do
    UPCASE=`echo "$i" | tr [:lower:] [:upper:]`
    echo "dep_tristate '  $i' CONFIG_QSEARCH_$UPCASE \$CONFIG_QSEARCH"
  done
  cat $TMPL_DIR/Config.in.part2
) > $KERNEL_DIR/qsearch/Config.in

for plugin in $PLUGINS
do
    echo "Installing plugin $plugin"
    PLUGIN=`echo "$plugin" | tr [:lower:] [:upper:]`

    mkdir -p $KERNEL_DIR/qsearch/$plugin
    cp $PLUGINS_DIR/$plugin/$plugin.[ch] $KERNEL_DIR/qsearch/$plugin
    sed -e "s/@PLUGIN_NAME@/$plugin/g"    \
        -e "s/@PLUGIN_NAME_UP@/$PLUGIN/g" \
       < $TMPL_DIR/Makefile.plugin > $KERNEL_DIR/qsearch/$plugin/Makefile
    sed -e "s/@PLUGIN_NAME@/$plugin/g"    \
       < $TMPL_DIR/qsearch_plugin.c      \
       > $KERNEL_DIR/qsearch/$plugin/qsearch_$plugin.c
done
    

