/*
 * skeleton.c -- skeleton of a plugin for the qsearch API.
 *               see http://www.cartel-securite.net/pbiondi/libqsearch.html
 *               for more informations
 *
 * Copyright (C) 2002  Philippe Biondi <biondi@cartel-securite.fr>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 */


/* qsearch.h is the only authorized (and mandatory) 
 * header file. It will include eveything we need
 * wether we are in user space or not.
 */

#include "qsearch.h"

typedef qsearch_state_t skeleton_state_t;
typedef struct skeleton_algo_s    skeleton_algo_t;
typedef struct skeleton_obj_s     skeleton_obj_t;
typedef struct skeleton_options_s skeleton_options_t;

struct skeleton_algo_s {
        QSEARCH_ALGO(skeleton)
};

struct skeleton_options_s {
        QSEARCH_OPTIONS(skeleton)
};

struct skeleton_obj_s {
	QSEARCH_OBJ(skeleton);
};

/*
 * Constructor
 */
static skeleton_obj_t *skeleton_constructor(skeleton_algo_t *algo, skeleton_options_t *options)
{
	skeleton_obj_t *self;
	self=malloc(sizeof(skeleton_obj_t));
	if (! self) 
		return NULL;
	self->statics=algo;

	/* Put here initialization of other members of alg data structure */

	return self;
}

/*
 * Destructor
 */
static int skeleton_destructor(skeleton_obj_t *self)
{
	/* Free here allocated memory */
	free(self);
	return 0;
}

/*
 * Add pattern
 */
static int skeleton_add_pattern(skeleton_obj_t *self, unsigned char *pattern, int len, void *ptrn_data, int type)
{
	return 0;
}

/*
 * Compile
 */
static int skeleton_compile(skeleton_obj_t *self)
{
	return 0;
}

/*
 * Search
 */
static int skeleton_search(skeleton_obj_t *self, 
			   skeleton_state_t *state_in, skeleton_state_t **state_out,
			   skeleton_state_t *state_io, 
			   qsearch_callback_t cb, void *cb_data,
			   void *buffer, size_t len)
{
	return 0;
}

/*
 * Alloc IO state
 */
static int skeleton_alloc_io_state(skeleton_obj_t *self, skeleton_state_t **state)
{
  	return 0;
}

/*
 * Free state
 */
static int skeleton_free_state(skeleton_obj_t *self, skeleton_state_t **state)
{
  	return 0;
}


/*
 * Structures
 */
static skeleton_algo_t algo_skeleton = {
	"skeleton",
	"This is an example",
	1<<QSEARCH_CS | 1<<QSEARCH_CI | 1<<QSEARCH_JOKER_CS | 1<<QSEARCH_JOKER_CI ,
	&skeleton_constructor,
	&skeleton_destructor,
	&skeleton_add_pattern,
	&skeleton_compile,
	&skeleton_search,
	&skeleton_alloc_io_state,
	&skeleton_free_state,
};

skeleton_algo_t *QSEARCH_ALGO_LIST_SYM[] = {
	&algo_skeleton,
	NULL
};

int QSEARCH_API_VERSION_SYM = QSEARCH_API_VERSION;
