%{
/*
 * readconf-grammar.y -- Grammar for the qsearch testsuite config file.
 * see http://www.cartel-securite.net/pbiondi/libqsearch.html
 * for more informations
 *
 * Copyright (C) 2002  Philippe Biondi <biondi@cartel-securite.fr>
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation; either version 2, or (at your option) any
 * later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 */


#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "testsuite.h"
#include "qsearch.h"

extern FILE *yyin;

int yyparse(void);
int yylex(void);


static void yyerror(const char *str)
{
        fprintf(stderr,"Error reading config. file: [%s]\n",str);
}
 

static char *info;
static char *sample;
static int resnb;
static char **resstr;
static int patnb;
static ptrn_t **patlist;

static test_t **testlist;
static int testnb;

static test_t *test;
static ptrn_t *pat;

static void reset_vars(void)
{
 	info="";
	sample=NULL;
	patnb=0;
	patlist=NULL;
	resnb=0;
	resstr=NULL;
}
 
test_t **readconf(char *filename)
{
	yyin = fopen(filename, "r" );
	if (! yyin) {
		fprintf(stderr,"Can't open file [%s]\n", filename);
		return NULL;
	}

	testnb=0;
	testlist=NULL;

	reset_vars();

        yyparse();
	
	return testlist;

} 

%}

%union {
        char *str;
        int number;
}

%token EXACTNESSTOK SPEEDTOK CSTOK CITOK JOKERTOK JOKERCITOK 
%token REGEXTOK SAMPLETOK INFOTOK PATTERNTOK RESULTSTOK
%token OBRACE EBRACE COLON EQUALS SEMICOLON COMMA
%token <str> STRING
%type <number> type

%start tests

%%
tests: /* empty */
        | tests test SEMICOLON
	;

test:
    	exactnesstest | speedtest

speedtest: SPEEDTOK STRING 


exactnesstest:	
        EXACTNESSTOK STRING testset
        { 
		if (! (patlist && resstr && sample)) {
			fprintf(stderr,"missing a mandatory field in test [%s]\n",(char *)$2);
			return -1;
		}
		else {
			test=malloc(sizeof(test_t));
			test->name=$2;
			test->info=info;
			test->sample=sample;
			test->patterns=patlist;
			test->results=resstr;

			testnb++;
			testlist=realloc(testlist, (1+testnb)*sizeof(test_t *));
			testlist[testnb-1]=test;
			testlist[testnb]=NULL;
	
			reset_vars();

			DBG("Parsed test [%s]\n", $2); 
		}
	}
        ;

testset:
	OBRACE attribs EBRACE
	;

attribs: 
        | attribs attrib SEMICOLON
	;

attrib:
      	SAMPLETOK STRING
        { 
		sample=$2;
		DBG(" stored sample [%s]\n", $2); 
	}
	|
	INFOTOK STRING
        {
		info=$2;
		DBG(" stored info [%s]\n", $2);
	}
	|
	PATTERNTOK patternset
	|
	RESULTSTOK resultsset 
	;

patternset:
	pattern
	|
	OBRACE patterns EBRACE
	;

patterns:
	| patterns pattern SEMICOLON
	;

pattern: 
       	STRING COLON type
        { 
		patnb++;
		patlist=realloc(patlist, (1+patnb)*sizeof(ptrn_t *));
		pat=malloc(sizeof(ptrn_t));
		pat->pattern=$1;
		pat->type=$3;
		patlist[patnb-1]=pat;
		patlist[patnb]=NULL;
		
		DBG("   stored pattern [%s]\n", $1); 
	}
	;

type:
    	CSTOK { $$=QSEARCH_CS; } 
	| CITOK { $$=QSEARCH_CI; }
	| JOKERTOK { $$=QSEARCH_JOKER_CS; }
	| JOKERCITOK { $$=QSEARCH_JOKER_CI; }
	| REGEXTOK { $$=QSEARCH_REGEXP; }
	;

resultsset:
	result
	|
	OBRACE results EBRACE
	;

results:
     	| results result SEMICOLON
	;

result:
      	STRING 
        { 
		resnb++;
		resstr=realloc(resstr, (1+resnb)*sizeof(char *));
		resstr[resnb-1]=$1;
		resstr[resnb]=NULL;
         	DBG("   stored result [%s]\n", $1); 
	}
	;

%%
